@@ -98,6 +98,24 @@ class MadFile(fantail.Fanstack):
         #call when the file has been is deleted
         pass
 
+    def flush(self):
+        """
+        Flush stores
+        """
+        self.hook_method('flush')
+        for s in self.stores:
+            store = self.stores[s]
+            store.flush()
+
+    def delete(self):
+        """
+        """
+        self.hook_method('madfile_delete', self)
+        for s in self.stores:
+            store = self.stores[s]
+            store.delete(self)
+
+
     def load(self, sha1sum=None):
         """
         load the record from the database, possibly with an
@@ -108,21 +126,52 @@ class MadFile(fantail.Fanstack):
             if os.path.exists(self.all['inputfile']) \
             else True
 
-        self.hook_method('madfile_pre_load', self)
+        if sha1sum is None:
+            # if a sha1sum is specified - do not call hooks
+            # just get the data from the storage
+            self.hook_method('madfile_pre_load', self)
 
-        for s in self.stores:
-            store = self.stores[s]
-            store.load(self, sha1sum=sha1sum)
+        fis = self.get('filesize', -1)
+        if fis < 1:
+            lg.info("file size zero, skip: {}".format(self['inputfile']))
+        else:
+            for s in self.stores:
+                store = self.stores[s]
+                store.load(self, sha1sum=sha1sum)
 
-        self.hook_method('madfile_load', self)
-        self.hook_method('madfile_post_load', self)
+        if sha1sum is None:
+            self.hook_method('madfile_load', self)
+            self.hook_method('madfile_post_load', self)
 
     def save(self):
         self.hook_method('madfile_save', self)
         self.hook_method('madfile_pre_save', self)
 
-        for s in self.stores:
+        fis = self.get('filesize', -1)
+        if fis <= 0:
+            lg.info("file size zero - not annotating this")
+        else:
+            for s in self.stores:
                 store = self.stores[s]
                 store.save(self)
 
         self.hook_method('madfile_post_save', self)
+
+
+
+class MadDummy(MadFile):
+     def __init__(self, data_core, data_all,
+                 stores=None,
+                 hook_method=dummy_hook_method):
+
+        self.stores = stores
+        self.hook_method = hook_method
+
+        super(MadFile, self).__init__(
+            stack=[data_core, data_all])
+
+        lg.debug("Instantiating a dummy madfile")
+
+        for s in self.stores:
+            store = self.stores[s]
+            store.prepare(self)
