@@ -10,7 +10,7 @@ lg = logging.getLogger(__name__)
 @leip.hook("madfile_post_load", 100)
 def lbconthefly(app, madfile):
 
-    fullpath = madfile.all.fullpath
+    fullpath = madfile['fullpath']
     mtcher = re.compile(r'/media/(seq-srv-[0-9][0-9])', re.I)
     mtch = mtcher.search(fullpath)
     if not mtch:
@@ -19,9 +19,7 @@ def lbconthefly(app, madfile):
 
     server = mtch.groups()[0]
 
-    madfile.all.host = server.upper()
-    #fp = madfile.all.fullpath
-    #fp = fp[fp.index(server) + len(server):]
-    #dn = os.path.dirname(fp)
-    #madfile.all.fullpath = fp
-    #madfile.all.dirname = dn
+    #TDDO: remove this later - bugfix
+    if 'host' in madfile.mad:
+        del madfile.mad['host']
+    madfile.all['host'] = server.upper()
