@@ -3,6 +3,7 @@ from __future__ import print_function
 import cPickle
 import collections
 import datetime
+import glob
 import logging
 import os
 import re
@@ -14,11 +15,10 @@ import hashlib
 
 import arrow
 from pymongo import MongoClient
+import pymongo
 from termcolor import cprint
 import yaml
 
-import pandas as pd
-
 import leip
 
 from mad2.util import get_all_mad_files, humansize, persistent_cache
@@ -101,11 +101,13 @@ def mongo_prep_mad(mf):
 MONGO_SAVE_CACHE = []
 MONGO_REMOVE_CACHE = []
 
+@leip.hook("flush") #can call this function as a hook!
 def mongo_flush(app):
 
     global MONGO_REMOVE_CACHE
     global MONGO_SAVE_CACHE
 
+    lg.debug("flush")
     if (len(MONGO_SAVE_CACHE) + len(MONGO_REMOVE_CACHE)) == 0:
         return
 
@@ -145,7 +147,8 @@ def save_to_mongo(app, madfile):
         #print(newrec['host'])
         MONGO_SAVE_CACHE.append((mongo_id, newrec))
 
-    if len(MONGO_SAVE_CACHE)  + len(MONGO_REMOVE_CACHE) > 33:
+    lg.debug("prep for save: %s", madfile['inputfile'])
+    if len(MONGO_SAVE_CACHE)  + len(MONGO_REMOVE_CACHE) > 20:
         mongo_flush(app)
 
 
@@ -204,17 +207,15 @@ def mongo_get(app, args):
     """
     get a mongodb record based on id
     """
-    MONGO_D = get_mongo_db(app)
-
-    mongo_id = args.mongo_id
-
-    rec = MONGO_D.find_one({'_id': mongo_id})
 
     if args.core:
          MONGO_C = get_mongo_core_db(app)
          core_id = rec['sha1sum'][:24]
          rec = MONGO_C.find_one({'_id': core_id})
-
+    else:
+        MONGO_D = get_mongo_db(app)
+        mongo_id = args.mongo_id
+        rec = MONGO_D.find_one({'_id': mongo_id})
 
     if not rec:
         return
@@ -237,6 +238,14 @@ def mongo_del(app, args):
     MONGO.remove({'_id': mongo_id})
 
 
+@leip.hook("madfile_delete")
+def dump_delete(app, madfile):
+    dump_id = madfile.get('_id_dump')
+    MONGO = get_mongo_db(app)
+    lg.debug("Deleting %s (%s)", madfile['inputfile'], dump_id)
+    MONGO.remove({'_id': dump_id})
+
+
 @leip.subcommand(mongo, "count")
 def mongo_count(app, args):
     """
@@ -263,42 +272,20 @@ def mongo_last(app, args):
               r['filename'], r.get('_id', '')]))
 
 
-@leip.flag('-e', '--echo')
-@leip.arg('file', nargs="*")
-@leip.command
-def repl(app, args):
-    """
-    Save to mongodb
-    """
-
-    MONGO_mad = get_mongo_db(app)
-
-    backup_hosts = set()
-    for host in app.conf['host']:
-        if app.conf['host'][host]['backup']:
-            backup_hosts.add(host)
-    for madfile in get_all_mad_files(app, args):
-        #print(madfile['sha1sum'])
-        query = {'sha1sum': madfile['sha1sum']}
-        res = MONGO_mad.find(query)
-        for r in res:
-            days = (arrow.now() - arrow.get(r['save_time'])).days
-            cprint('%1d' % r['nlink'], 'yellow', end=" ")
-            cprint('%4d' % days, 'green', end="d ")
-            if r['host'] in backup_hosts:
-                cprint(r['host'], 'green', attrs=['bold'], end=':')
-            else:
-                cprint(r['host'], 'cyan', end=':')
-            cprint(r['fullpath'])
-
-
 @leip.flag('--delete')
 @leip.arg('-u', '--username')
 @leip.arg('-b', '--backup')
 @leip.arg('-B', '--ignore_backup_volumes')
 @leip.arg('-v', '--volume')
+@leip.arg('-p', '--project')
+@leip.arg('-P', '--pi')
+@leip.arg('-e', '--experiment')
 @leip.arg('-H', '--host')
 @leip.arg('-s', '--sha1sum')
+@leip.arg('-S', '--sort', help='sort on this field')
+@leip.arg('-R', '--reverse_sort', help='reverse sort on this field')
+@leip.arg('-l', '--limit', default=-1, type=int)
+@leip.arg('-f', '--format', help='output format', default='{fullpath}')
 @leip.command
 def search(app, args):
     """
@@ -310,7 +297,7 @@ def search(app, args):
     query = {}
 
     for f in ['username', 'backup', 'volume', 'host',
-              'sha1sum']:
+              'sha1sum', 'project', 'project', 'pi']:
         if not f in args:
             continue
 
@@ -324,8 +311,18 @@ def search(app, args):
         return
 
     res = MONGO_mad.find(query)
+
+    if args.sort:
+        res = res.sort(args.sort, pymongo.ASCENDING)
+    elif args.reverse_sort:
+        res = res.sort(args.reverse_sort, pymongo.DESCENDING)
+
+    if args.limit > 0:
+        res = res.limit(args.limit)
+    
     for r in res:
-        print(r['fullpath'])
+
+        print(args.format.format(**r)) #['fullpath'])
 
 
 @persistent_cache(leip.get_cache_dir('mad2', 'mongo', 'sum'),
@@ -335,6 +332,7 @@ def _single_sum(app, group_by=None, force=False):
     MONGO_mad = get_mongo_db(app)
 
     res = MONGO_mad.aggregate([
+        {"$match" : {"orphan" : False}},
         {'$group': {
             "_id": groupby_field,
             "total": {"$sum": "$filesize"},
@@ -412,6 +410,7 @@ def mongo_sum2(app, args):
         sort_field = 'total'
         sort_order = -1
     res = MONGO_mad.aggregate([
+            {"$match" : {"orphan" : False}},
             {'$group': {
                 "_id": {
                     "group1": gb1_field,
@@ -428,7 +427,6 @@ def mongo_sum2(app, args):
     gl1 = gl2 = len("Total")
 
     for r in res['result']:
-        #print(r)
         g1 = str(r['_id'].get('group1'))
         g2 = str(r['_id'].get('group2'))
         gl1 = max(gl1, len(g1))
@@ -484,7 +482,8 @@ def complex_sum(app, args):
         'username',
         'experiment_type',
         'host',
-        'volume,',
+        'volume',
+        'orphan',
         'filetype',
         'organism',
         'biologist',
@@ -512,10 +511,88 @@ def complex_sum(app, args):
         for f in fields:
             prec[f] = _id.get(f)
         precords.append(prec)
-    df = pd.DataFrame(precords)
 
     out = args.output_file.format(stamp=stamp)
-    df.to_csv(out, sep="\t", encoding="utf8")
+    with open(out, 'w') as F:
+        F.write(yaml.safe_dump(precords, default_flow_style=False))
+
+
+def get_latest_csum(app, args):
+
+    import pandas as pd
+
+    #select the correct csum file
+    csum_path = app.conf['plugin.mongo.csum_path']
+    assert '{stamp}' in csum_path
+    csum_glob = csum_path.replace('{stamp}', '*')
+    csums = []
+    css = csum_path.index('{stamp}')
+    for f in glob.glob(csum_glob):
+        cdate = f[css:].split('.')[0]
+        csums.append((cdate, f))
+    csums.sort()
+    cdate, csum_file = csums[-1]
+    lg.info("Using csum file: %s", csum_file)
+    lg.info("From date: %s", cdate)
+
+    pickle_file = csum_file.replace('.yaml', '') + '.pickle'
+    if os.path.exists(pickle_file):
+        data = pd.read_pickle(pickle_file)
+        return data
+
+    with open(csum_file) as F:
+        data = yaml.load(F)
+
+    data = pd.DataFrame(data)
+    data.fillna('n.a.', inplace=True)
+
+    data.to_pickle(pickle_file)
+    return data
+
+@leip.arg("-s", "--select", nargs=2, action='append')
+@leip.arg("group", nargs='+')
+@leip.subcommand(mongo, "csum_group")
+def mongo_csum_report(app, args):
+
+    import pandas as pd
+
+    pd.set_option('display.max_rows', None)
+
+    data = get_latest_csum(app, args)
+
+    if not args.select is None:
+        for selkey, selval in args.select:
+            data = data[data[selkey] == selval]
+
+    grod = data.groupby(args.group)[['sum', 'count']].sum()
+    grod.sort('sum', inplace=True, ascending=False)
+    grod['sum'] = grod['sum'].apply(humansize)
+    print(grod)
+
+
+
+@leip.arg("-s", "--select", nargs=2, action='append')
+@leip.arg('values', nargs='*', default='sum')
+@leip.arg("columns")
+@leip.arg("index")
+@leip.subcommand(mongo, "csum_pivot")
+def mongo_csum_pivot(app, args):
+
+    import pandas as pd
+    pd.set_option('display.max_rows', None)
+
+    data = get_latest_csum(app, args)
+    print(data.head())
+
+    if not args.select is None:
+        for selkey, selval in args.select:
+            data = data[data[selkey] == selval]
+
+    piv = data.pivot_table(index=args.index, columns=args.columns,
+                     values=args.values)
+    piv = piv.applymap(humansize)
+    piv.fillna("", inplace=True)
+    print(piv)
 
 
 @leip.flag('--remove-from-core', help='also remove from the core db')
@@ -546,6 +623,7 @@ def forget(app, args):
     go(MONGO_CORE, to_remove_core)
 
 
+@leip.flag('-r', '--remove-from-dump', help='remove from the dump db')
 @leip.flag('--remove-from-core', help='also remove from the core db')
 @leip.flag('--run', help='actually run - otherwise it\'s a dry run showing ' +
            'what would be deleted')
@@ -557,6 +635,8 @@ def forget_dir(app, args):
     flush_dir(app, args)
 
 
+@leip.flag('-r', '--remove-from-dump', help='remove from the dump db '
+           + 'otherwise files in dump are tagged as deleted')
 @leip.flag('--remove-from-core', help='also remove from the core db')
 @leip.flag('--run', help='actually run - otherwise it\'s a dry run showing ' +
            'what would be deleted')
@@ -570,6 +650,11 @@ def flush_dir(app, args):
     Recursively flush deleted files from the dump db
     """
 
+    if args.remove_from_core and not args.remove_from_dump:
+        lg.warning("This is an odd command to give - why remove from the core")
+        lg.warning("  and not the dump db?")
+        exit()
+
     MONGO_mad = get_mongo_db(app)
     MONGO_core = get_mongo_core_db(app)
 
@@ -580,7 +665,8 @@ def flush_dir(app, args):
 
     query = {
         'host' : host,
-        'dirname' : rex}
+        'dirname' : rex,
+        'orphan': False}
 
     ids_to_remove = []
     core_to_remove = []
@@ -593,19 +679,31 @@ def flush_dir(app, args):
 
         if os.path.exists(r['fullpath']):
             if args.echo:
-                print("+ " + r['fullpath'])
+                try:
+                    print("+ " + r['fullpath'])
+                except UnicodeEncodeError:
+                    print("+ " + r['fullpath'].encode('utf8'))
+
             if not args.forget_all:
                 continue
         else:
-            print("- " + r['fullpath'])
+            try:
+                print("- " + r['fullpath'])
+            except UnicodeEncodeError:
+                print("- " + r['fullpath'].encode('utf8'))
 
         ids_to_remove.append(r['_id'])
+
         if args.remove_from_core:
             core_to_remove.append(r['sha1sum'][:24])
 
         if args.run and len(ids_to_remove) >= 100:
-            lg.warning("removing %d records", len(ids_to_remove))
-            MONGO_mad.remove( {'_id' : { '$in' : ids_to_remove } } )
+            if args.remove_from_dump:
+                lg.warning("removing %d records", len(ids_to_remove))
+                MONGO_mad.remove( {'_id' : { '$in' : ids_to_remove } } )
+            else:
+                lg.warning("setting %d records to orphaned", len(ids_to_remove))
+
             ids_to_remove = []
 
         if args.run and args.remove_from_core and \
@@ -616,14 +714,23 @@ def flush_dir(app, args):
             core_to_remove = []
 
     if args.run:
-        lg.warning("removing %d records", len(ids_to_remove))
-        MONGO_mad.remove( {'_id' : { '$in' : ids_to_remove } } )
+        if args.remove_from_dump:
+            lg.warning("removing %d records", len(ids_to_remove))
+            MONGO_mad.remove( {'_id' : { '$in' : ids_to_remove } } )
+        else:
+            lg.warning("setting %d records to orphaned", len(ids_to_remove))
+            MONGO_mad.update(
+                {'_id' : { '$in' : ids_to_remove } },
+                {'$set': {'orphan': True}}
+                )
 
     if args.run and args.remove_from_core:
             lg.warning("removing %d records from core",
                        len(core_to_remove))
             MONGO_core.remove( {'_id' : { '$in' : core_to_remove } } )
 
+
+
 @leip.flag('-f', '--force')
 @leip.subcommand(mongo, "drop")
 def mongo_drop(app, args):
@@ -658,9 +765,12 @@ def mongo_index(app, args):
     Ensure indexes on the relevant fields
     """
     MONGO_mad = get_mongo_db(app)
+    MONGO_core = get_mongo_core_db(app)
+
     for f in app.conf['plugin.mongo.indici']:
         print("create index on: {}".format(f))
         MONGO_mad.ensure_index(f)
+        MONGO_core.ensure_index(f)
 
 
 @persistent_cache(leip.get_cache_dir('mad2', 'mongo', 'keys'),
@@ -716,11 +826,9 @@ def mongo_keys(app, args):
         print( ('%-' + str(max_key_len) + 's: %12s %12s') % (
               k, dump.get(k, ''), core.get(k, '')))
 
-    df = pd.DataFrame(records)
-    df.set_index('key', inplace=True)
-
     out = args.output_file.format(stamp=stamp)
-    df.to_csv(out, sep="\t", encoding="utf8")
+    with open(out, 'w') as F:
+        F.write(yaml.safe_dump(records, default_flow_style=False))
 
 
 @leip.arg('keyname', metavar='key')
@@ -763,15 +871,16 @@ def _run_mongo_command(app, name, collection, query, kwargs={}, force=False):
     """
     MONGO_mad = get_mongo_db(app)
     res = MONGO_mad.database.command(query, collection, **kwargs)
-
     return res
 
 
+
 WASTE_PIPELINE = [
-    {"$sort": {"sha1sum": 1 } },
+    {"$match" : {"orphan" : False}},
+    {"$sort": {"sha1sum": 1 }},
     {"$project": {"filesize": 1,
-                   "sha1sum": 1,
-                   "usage": {"$divide": ["$filesize", "$nlink"]}}},
+                  "sha1sum": 1,
+                  "usage": {"$divide": ["$filesize", "$nlink"]}}},
     {"$group": {"_id": "$sha1sum",
                  "no_records": {"$sum": 1 },
                  "mean_usage": {"$avg": "$usage" },
@@ -787,14 +896,228 @@ WASTE_PIPELINE = [
                 "waste": {"$sum": "$waste" }}}]
 
 
+@leip.flag('-e', '--echo')
+@leip.arg('file', nargs="*")
+@leip.command
+def repl(app, args):
+    """
+    Save to mongodb
+    """
 
+    MONGO_mad = get_mongo_db(app)
+
+    backup_hosts = set()
+    for host in app.conf['host']:
+        if app.conf['host'][host]['backup']:
+            backup_hosts.add(host)
+
+
+    check_shasums = False
+    if args.file > 0:
+        for f in args.file:
+            if os.path.exists(f): break
+            if len(f) != 40: break
+        else: check_shasums = True
+
+    def _process_query(query):
+        res = MONGO_mad.find(query)
+        for r in res:
+            days = (arrow.now() - arrow.get(r['save_time'])).days
+            symlink = r.get('is_symlink', False)
+            if symlink:
+                stag = 'S'
+            else:
+                stag = '.'
+            cprint('%1d%s' % (r['nlink'], stag), 'yellow', end=" ")
+            cprint('%3d' % days, 'green', end="d ")
+            cprint('%6s' % humansize(r['filesize']), 'white', end=" ")
+            if r['host'] in backup_hosts:
+                cprint(r['host'], 'green', attrs=['bold'], end=':')
+            else:
+                cprint(r['host'], 'cyan', end=':')
+            cprint(r['fullpath'])
+
+    if check_shasums:
+        for sha1sum in args.file:
+            query = {'sha1sum' : sha1sum}
+            _process_query(query)
+    else:
+        for madfile in get_all_mad_files(app, args):
+            #print(madfile['sha1sum'])
+            query = {'sha1sum': madfile['sha1sum']}
+            _process_query(query)
+
+
+
+FIND_WASTER_PIPELINE = [
+    {"$match" : {"orphan" : False}},
+    {"$project": {"filesize": 1,
+                  "sha1sum": 1,
+                  "usage": {"$divide": ["$filesize", "$nlink"]}}},
+    {"$group": {"_id": "$sha1sum",
+                "no_records": {"$sum": 1 },
+                "mean_usage": {"$avg": "$usage" },
+                "total_usage": {"$sum": "$usage" },
+                "filesize": {"$max": "$filesize" }}},
+    {"$project": {"filesize": 1,
+                  "total_usage": 1,
+                  "waste": {"$subtract": ["$total_usage", "$filesize"]}}},
+    {"$match": {"waste": {"$gt": 500}}},
+    {"$sort" : {"waste": -1} },
+    {"$limit": 100}]
+
+
+@persistent_cache(leip.get_cache_dir('mad2', 'mongo', 'waste'), 1,  24*60*60)
+def _run_waste_command(app, name, force=False):
+    """
+    Execute mongo command.
+    """
+    MONGO_mad = get_mongo_db(app)
+    res = MONGO_mad.aggregate(FIND_WASTER_PIPELINE, allowDiskUse=True)
+    return res
+
+
+@leip.flag('-N', '--no-color', help='no ansi coloring of output')
+@leip.arg('-n', '--no-records', default=20, type=int)
 @leip.flag('-f', '--force')
 @leip.command
 def waste(app, args):
 
-    mongo_info = app.conf['plugin.mongo']
-    coll = mongo_info.get('collection', 'mad2')
+    db = get_mongo_db(app)
 
-    print(WASTE_PIPELINE)
+    res = _run_waste_command(app, 'waste_pipeline',
+                             force=args.force)['result']
+
+    for i, r in enumerate(res):
+        if i >= args.no_records:
+            break
+
+        sha1sum = r['_id']
+        if not sha1sum.strip():
+            continue
+
+        cprint(sha1sum, 'grey', end='')
+        cprint(" sz ", "grey", end="")
+        cprint("{:>9}".format(humansize(r['waste'])), end='')
+        cprint(" w ", "grey", end="")
+        cprint("{:>9}".format(humansize(r['filesize'])),end='')
+
+        hostcount = collections.defaultdict(lambda: 0)
+        hostsize = collections.defaultdict(lambda: 0)
+        owners = set()
+        for f in db.find({'sha1sum': sha1sum}):
+            owners.add(f['username'])
+            host = f['host']
+            hostcount[host] += 1
+            hostsize[host] += float(f['filesize']) / float(f['nlink'])
+
+        for h in hostcount:
+            print(' ', end='')
+            cprint(h, 'green', end=':')
+            cprint(hostcount[h], 'cyan', end="")
+
+        cprint(" ", end="")
+        cprint(", ".join(owners), 'red')
 
 
+@leip.arg('-S', '--subject')
+@leip.flag('-f', '--force')
+@leip.command
+def waste_text_report(app, args):
+
+    db = get_mongo_db(app)
+
+    res = _run_waste_command(app, 'waste_pipeline',
+                             force=args.force)['result']
+
+
+    if args.subject:
+        print("Subject: {}".format(args.subject))
+
+
+    #This week's winner
+    top = res[0]
+    sha1sum = top['_id']
+    owners = set()
+    hostcount = collections.defaultdict(lambda: 0)
+    hostsize = collections.defaultdict(lambda: 0)
+
+
+    total = 0
+    for rec in db.find({'sha1sum': sha1sum}):
+        total += 1
+        host = rec['host']
+        hostcount[host] += 1
+        hostsize[host] += float(rec['filesize']) / float(rec['nlink'])
+        owners.add(rec['username'])
+
+
+
+    print("This week's winner: {}".format(", ".join(owners)))
+    print("One file, ", end="")
+    print("{} location".format(total), end="")
+    if total > 1: print("s", end="")
+    print(", {} server,".format(len(hostcount)), end="")
+    if len(hostcount) > 1: print("s", end="")
+    print(" wasting {}.".format(humansize(top['waste'])))
+    print("try:\n   mad repl {}\n\n".format(sha1sum))
+
+    no_to_print = 20
+    print("Waste overview: (no / sha1sum / waste / filesize)")
+    print("=================================================\n")
+    for i, r in enumerate(res):
+        if i >= no_to_print:
+            break
+
+        sha1sum = r['_id']
+        if not sha1sum.strip():
+            continue
+        print("{:2d} {} {:>10} {:>10}"\
+                    .format(i, sha1sum, humansize(r['waste']),
+                            humansize(r['filesize'])))
+
+
+    print("\n\nDetails: (nlink/symlink/size/owner)")
+    print("===================================")
+    import textwrap
+    for i, r in enumerate(res):
+        if i >= no_to_print:
+            break
+
+        sha1sum = r['_id']
+        if not sha1sum.strip():
+            continue
+
+        print("# {:2d} {} {:>10} {:>10}"\
+                    .format(i, sha1sum, humansize(r['waste']),
+                            humansize(r['filesize'])))
+
+        records = collections.defaultdict(list)
+        hostcount = collections.defaultdict(lambda: 0)
+        hostsize = collections.defaultdict(lambda: 0)
+
+        for rec in db.find({'sha1sum': sha1sum}):
+            host = rec['host']
+            records[host].append(rec)
+            hostcount[host] += 1
+            hostsize[host] += float(rec['filesize']) / float(rec['nlink'])
+
+
+        for h in hostcount:
+            print("## Host: {}, copies: {}, total use: {}".format(
+                      h, hostcount[h], humansize(hostsize[h])))
+            for rec in records[host]:
+                smarker = '.'
+                if rec.get('is_symlink'):
+                    smarker = 'S'
+                print("  {} {}".format(rec.get('nlink', '?'), smarker),
+                      end=' ')
+                print(humansize(rec['filesize']), end=' ')
+                print(rec['username'])
+                print("   " + rec['fullpath'])
+#                for j, pp in enumerate(textwrap.wrap(rec['fullpath'], 70)):
+#                    print(" " * 8 + pp)
+
+
+        print("")
+
