@@ -8,14 +8,13 @@ import errno
 import logging
 import os
 import re
-import select
 import sys
 import time
 
 from termcolor import cprint
 
 from mad2.exception import MadPermissionDenied, MadNotAFile
-from mad2.madfile import MadFile
+from mad2.madfile import MadFile, MadDummy
 
 import fantail
 
@@ -131,6 +130,33 @@ def get_mad_file(app, filename):
                    hook_method=app.run_hook)
 
 
+def get_mad_dummy(app, data):
+    """
+    instantiate a dummy - only used to save.
+
+    """
+
+    global STORES
+    if STORES is None:
+        initialize_stores(app)
+
+    if 'madname' in data:
+        del data['madname']
+    if 'fullmadpath' in data:
+        del data['fullmadpath']
+
+    data_all = fantail.Fantail(data)
+    data_core = fantail.Fantail()
+
+    for kw in data_all.keys():
+        tra = app.conf['keywords'][kw].get('transient', False)
+        if not tra:
+            data_core[kw] = data_all[kw]
+
+    lg.debug("instantiating dummy madfile")
+    return MadDummy(data_all=data_all, data_core=data_core, stores=STORES,
+                    hook_method=app.run_hook )
+
 def to_mad(fn):
     if '/' in fn:
         a, b = fn.rsplit('/')
@@ -153,6 +179,7 @@ def get_filenames(args, use_stdin=True, allow_dirs=False):
             return m.group('fn')
 
     if 'file' in args and len(args.file) > 0:
+
         for f in args.file:
             if len(f) == 0:
                 continue
@@ -165,10 +192,12 @@ def get_filenames(args, use_stdin=True, allow_dirs=False):
             if 'QDSUMS' in f:
                 continue
 
+
             if not os.access(f, os.R_OK):
                 # no read access - ignore this file
                 continue
 
+
             try:
                 os.stat(f)
             except OSError, e:
@@ -179,6 +208,8 @@ def get_filenames(args, use_stdin=True, allow_dirs=False):
                 else:
                     raise
 
+            lg.debug("processing %s", f)
+
             rv = demad.sub(demadder, f)
 
             if not allow_dirs and os.path.isdir(rv):
@@ -225,14 +256,17 @@ def get_all_mad_files(app, args, use_stdin=True, warn_on_errors=True):
 
 
 def humansize(nbytes):
+    import numpy as np
+    if np.isnan(nbytes):
+        return float("nan")
     suffixes = [' b', 'kb', 'Mb', 'Gb', 'Tt', 'Pb']
     if nbytes == 0:
-        return '0  b'
+        return '0 b'
     i = 0
     while nbytes >= 1024 and i < len(suffixes) - 1:
         nbytes /= 1024.
         i += 1
-    f = ('%.2f' % nbytes).rstrip('0').rstrip('.')
+    f = ('%.2f' % nbytes).rstrip('.')
     return '%s %s' % (f, suffixes[i])
 
 
